! *************************************************************************
! *
! *                    subroutine ludcmp(a,n,np,indx,d)
! *
! *  This is stolen directly from numerical recipes.
! *  As documented there it uses implicit pivoting
! *  I've modified it slightly to allocate/deallocate the scaling vector
! *
! *************************************************************************

subroutine ludcmp(a,n,np,indx,d)
  use precision
  implicit none
!
! --- parameters
!
  real(long),parameter :: tiny=1.0d-20
! 
! --- calling arguments
!
  integer n,np
  integer indx(n)
  
  real(long) :: d
  real(long) :: a(np,np)
!  
! --- local variables
!
  integer i,imax,j,k
  
  real(long) :: aamax,dum,sum
  real(long), allocatable,dimension(:) :: vv
!
  allocate(vv(np))
  d = 1.d0
  do i = 1,n
    aamax = 0.d0
    do j = 1,n
      if (abs(a(i,j)).gt.aamax) aamax = abs(a(i,j))
    enddo  
    if (aamax.eq.0.d0) stop 'singular matrix.'
    vv(i) = 1.d0/aamax
  enddo
  do j = 1,n
    if (j.gt.1) then
      do i = 1,j - 1
        sum = a(i,j)
        if (i.gt.1) then
          do k = 1,i - 1
            sum = sum - a(i,k)*a(k,j)
          enddo
          a(i,j) = sum
        end if
      enddo
    end if
!
    aamax = 0.d0
    do i = j,n
      sum = a(i,j)
      if (j.gt.1) then
        do k = 1,j - 1
          sum = sum - a(i,k)*a(k,j)
        enddo
        a(i,j) = sum
      end if
      
      dum = vv(i)*abs(sum)
      if (dum.ge.aamax) then
        imax = i
        aamax = dum
      end if
    enddo
! Comment out this block to switch off pivoting    
    if (j.ne.imax) then
      do k = 1,n
        dum = a(imax,k)
        a(imax,k) = a(j,k)
        a(j,k) = dum
      enddo
      d = -d
      vv(imax) = vv(j)
    end if
    
    indx(j) = imax
! and change this to read indx(j)=j
    if (j.ne.n) then
      if (a(j,j).eq.0.) a(j,j) = tiny
      dum = 1.d0/a(j,j)
      do i = j + 1,n
        a(i,j) = a(i,j)*dum
      enddo
    end if
  enddo
  if (a(n,n).eq.0.) a(n,n) = tiny
  deallocate(vv)
  return
end
    
